@@ -9,8 +9,8 @@
 #
 package Test::Moose::More;
 our $AUTHORITY = 'cpan:RSRCHBOY';
-# git description: 0.035-4-g75693a2
-$Test::Moose::More::VERSION = '0.036'; # TRIAL
+# git description: 0.038-24-g07edb95
+$Test::Moose::More::VERSION = '0.039'; # TRIAL
 
 # ABSTRACT: More tools for testing Moose packages
 
@@ -97,7 +97,7 @@ sub meta_ok ($;$) {
 }
 
 
-sub does_ok {
+sub does_ok ($$;$) {
     my ($thing, $roles, $message) = @_;
 
     my $thing_meta = find_meta($thing);
@@ -105,6 +105,7 @@ sub does_ok {
     $roles     = [ $roles ] unless ref $roles;
     $message ||= _thing_name($thing, $thing_meta) . ' does %s';
 
+    BEGIN { warnings::unimport 'redundant' if $^V gt v5.21.1 }
     $tb->ok(!!$thing_meta->does_role($_), sprintf($message, $_))
         for @$roles;
 
@@ -112,7 +113,7 @@ sub does_ok {
 }
 
 
-sub does_not_ok {
+sub does_not_ok ($$;$) {
     my ($thing, $roles, $message) = @_;
 
     my $thing_meta = find_meta($thing);
@@ -120,6 +121,7 @@ sub does_not_ok {
     $roles     = [ $roles ] unless ref $roles;
     $message ||= _thing_name($thing, $thing_meta) . ' does not do %s';
 
+    BEGIN { warnings::unimport 'redundant' if $^V gt v5.21.1 }
     $tb->ok(!$thing_meta->does_role($_), sprintf($message, $_))
         for @$roles;
 
@@ -148,7 +150,7 @@ sub has_attribute_ok ($$;$) {
 }
 
 
-sub has_method_ok {
+sub has_method_ok ($@) {
     my ($thing, @methods) = @_;
 
     ### $thing
@@ -163,9 +165,9 @@ sub has_method_ok {
 }
 
 
-sub role_wraps_around_method_ok { unshift @_, 'around'; goto \&_role_wraps }
-sub role_wraps_before_method_ok { unshift @_, 'before'; goto \&_role_wraps }
-sub role_wraps_after_method_ok  { unshift @_, 'after';  goto \&_role_wraps }
+sub role_wraps_around_method_ok ($@) { unshift @_, 'around'; goto \&_role_wraps }
+sub role_wraps_before_method_ok ($@) { unshift @_, 'before'; goto \&_role_wraps }
+sub role_wraps_after_method_ok  ($@) { unshift @_, 'after';  goto \&_role_wraps }
 
 sub _role_wraps {
     my ($style, $thing, @methods) = @_;
@@ -184,7 +186,7 @@ sub _role_wraps {
 }
 
 
-sub requires_method_ok {
+sub requires_method_ok ($@) {
     my ($thing, @methods) = @_;
 
     ### $thing
@@ -199,7 +201,7 @@ sub requires_method_ok {
 }
 
 
-sub is_immutable_ok($;$) {
+sub is_immutable_ok ($;$) {
     my ($thing, $message) = @_;
 
     ### $thing
@@ -209,7 +211,7 @@ sub is_immutable_ok($;$) {
     return $tb->ok($meta->is_immutable, $message);
 }
 
-sub is_not_immutable_ok($;$) {
+sub is_not_immutable_ok ($;$) {
     my ($thing, $message) = @_;
 
     ### $thing
@@ -221,11 +223,11 @@ sub is_not_immutable_ok($;$) {
 
 
 # NOTE: deprecate at some point late 2015
-sub is_role  { goto \&is_role_ok  }
-sub is_class { goto \&is_class_ok }
+sub is_role  ($) { goto \&is_role_ok  }
+sub is_class ($) { goto \&is_class_ok }
 
-sub is_role_ok  { unshift @_, 'Role';  goto \&_is_moosey_ok }
-sub is_class_ok { unshift @_, 'Class'; goto \&_is_moosey_ok }
+sub is_role_ok  ($) { unshift @_, 'Role';  goto \&_is_moosey_ok }
+sub is_class_ok ($) { unshift @_, 'Class'; goto \&_is_moosey_ok }
 
 sub _is_moosey_ok {
     my ($type, $thing) =  @_;
@@ -236,15 +238,27 @@ sub _is_moosey_ok {
     $tb->ok(!!$meta, "$thing_name has a metaclass");
     return unless !!$meta;
 
-    return $tb->ok($meta->isa("Moose::Meta::$type"), "$thing_name is a Moose " . lc $type);
+    my $is_moosey = $meta->isa("Moose::Meta::$type");
+
+    # special check for class -- this will happen when, say, you're validating
+    # an attribute and it's a bog standard Moose::Meta::Attribute: strictly
+    # speaking its metaclass is Class::MOPish, but really,
+    # a Moose::Meta::Attribute is a Moose class.  Or arguably so.  Certainly
+    # in the context of what we're asking about here.  Better approaches to
+    # this welcomed as pull requests :)
+
+    $is_moosey ||= ($meta->name || q{}) =~ /^Moose::Meta::/
+        if $type eq 'Class';
+
+    return $tb->ok($is_moosey, "$thing_name is a Moose " . lc $type);
 }
 
 
 # NOTE: deprecate at some point late 2015
-sub is_anon     { goto \&is_anon_ok     }
-sub is_not_anon { goto \&is_not_anon_ok }
+sub is_anon     ($) { goto \&is_anon_ok     }
+sub is_not_anon ($) { goto \&is_not_anon_ok }
 
-sub is_anon_ok {
+sub is_anon_ok ($) {
     my ($thing, $message) = @_;
 
     my $thing_meta = find_meta($thing);
@@ -253,7 +267,7 @@ sub is_anon_ok {
     return $tb->ok(!!$thing_meta->is_anon, $message);
 }
 
-sub is_not_anon_ok {
+sub is_not_anon_ok ($) {
     my ($thing, $message) = @_;
 
     my $thing_meta = find_meta($thing);
@@ -263,9 +277,9 @@ sub is_not_anon_ok {
 }
 
 
-sub known_sugar() { qw{ has around augment inner before after blessed confess } }
+sub known_sugar () { qw{ has around augment inner before after blessed confess } }
 
-sub check_sugar_removed_ok($) {
+sub check_sugar_removed_ok ($) {
     my $t = shift @_;
 
     # check some (not all) Moose sugar to make sure it has been cleared
@@ -275,7 +289,7 @@ sub check_sugar_removed_ok($) {
 }
 
 
-sub check_sugar_ok($) {
+sub check_sugar_ok ($) {
     my $t = shift @_;
 
     # check some (not all) Moose sugar to make sure it has been cleared
@@ -286,9 +300,9 @@ sub check_sugar_ok($) {
 
 
 
-sub validate_thing { _validate_subtest_wrapper(\&_validate_thing_guts, @_) }
-sub validate_class { _validate_subtest_wrapper(\&_validate_class_guts, @_) }
-sub validate_role  { _validate_subtest_wrapper(\&_validate_role_guts,  @_) }
+sub validate_thing ($@) { _validate_subtest_wrapper(\&_validate_thing_guts, @_) }
+sub validate_class ($@) { _validate_subtest_wrapper(\&_validate_class_guts, @_) }
+sub validate_role  ($@) { _validate_subtest_wrapper(\&_validate_role_guts,  @_) }
 
 sub _validate_subtest_wrapper {
     my ($func, $thing, %args) = @_;
@@ -422,8 +436,8 @@ sub _validate_role_guts {
 }
 
 
-sub _validate_attribute { _validate_subtest_wrapper(\&__validate_attribute_guts,                 @_) }
-sub validate_attribute  { _validate_subtest_wrapper( \&_validate_attribute_guts, [shift, shift], @_) }
+sub _validate_attribute       { _validate_subtest_wrapper(\&__validate_attribute_guts,                 @_) }
+sub validate_attribute ($$@)  { _validate_subtest_wrapper( \&_validate_attribute_guts, [shift, shift], @_) }
 
 sub _validate_attribute_guts {
     my ($thingname, %opts) = @_;
@@ -457,7 +471,7 @@ sub __validate_attribute_guts {
     return _attribute_options_ok($att, %opts);
 }
 
-sub attribute_options_ok {
+sub attribute_options_ok ($$@) {
     my ($thing, $name, %opts) = @_;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
@@ -482,14 +496,20 @@ sub _role_attribute_options_ok {
     my $name                    = $att->name;
     my $thing_name              = _thing_name($name, $att);
 
-    # this much works, at least
-    if (exists $opts{coerce}) {
+    exists $opts{required} and delete $opts{required}
+        ? ok($att->is_required,  "$thing_name is required")
+        : ok(!$att->is_required, "$thing_name is not required")
+        ;
 
-        delete $opts{coerce}
-            ? ok( $att->should_coerce, "$thing_name should coerce")
-            : ok(!$att->should_coerce, "$thing_name should not coerce")
-            ;
-    }
+    exists $opts{lazy} and delete $opts{lazy}
+        ? ok($att->is_lazy,  "$thing_name is lazy")
+        : ok(!$att->is_lazy, "$thing_name is not lazy")
+        ;
+
+    exists $opts{coerce} and delete $opts{coerce}
+        ? ok( $att->should_coerce, "$thing_name should coerce")
+        : ok(!$att->should_coerce, "$thing_name should not coerce")
+        ;
 
     ### for now, skip role attributes: blessed $att
     return $tb->skip('cannot yet test role attribute layouts')
@@ -499,10 +519,6 @@ sub _role_attribute_options_ok {
 sub _class_attribute_options_ok {
     my ($att, %opts) = @_;
 
-    ### for now, skip role attributes: blessed $att
-    return $tb->skip('cannot yet test role attribute layouts')
-        if $att->isa('Moose::Meta::Role::Attribute');
-
     my @check_opts =
         qw{ reader writer accessor predicate default builder clearer };
     my @unhandled_opts = qw{ isa does handles traits };
@@ -535,13 +551,10 @@ sub _class_attribute_options_ok {
         is($att->$property, $value, "$thing_name option $property correct")
     };
 
-    if (my $is_required = delete $opts{required}) {
-
-        $is_required
-            ? ok($att->is_required,  "$thing_name is required")
-            : ok(!$att->is_required, "$thing_name is not required")
-            ;
-    }
+    exists $opts{required} and delete $opts{required}
+        ? ok($att->is_required,  "$thing_name is required")
+        : ok(!$att->is_required, "$thing_name is not required")
+        ;
 
     $check->($_) for grep { any(@check_opts) eq $_ } sort keys %opts;
 
@@ -557,21 +570,15 @@ sub _class_attribute_options_ok {
         delete $opts{init_arg};
     }
 
-    if (exists $opts{lazy}) {
+    exists $opts{lazy} and delete $opts{lazy}
+        ? ok($att->is_lazy,  "$thing_name is lazy")
+        : ok(!$att->is_lazy, "$thing_name is not lazy")
+        ;
 
-        delete $opts{lazy}
-            ? ok($att->is_lazy,  "$thing_name is lazy")
-            : ok(!$att->is_lazy, "$thing_name is not lazy")
-            ;
-    }
-
-    if (exists $opts{coerce}) {
-
-        delete $opts{coerce}
-            ? ok( $att->should_coerce, "$thing_name should coerce")
-            : ok(!$att->should_coerce, "$thing_name should not coerce")
-            ;
-    }
+    exists $opts{coerce} and delete $opts{coerce}
+        ? ok( $att->should_coerce, "$thing_name should coerce")
+        : ok(!$att->should_coerce, "$thing_name should not coerce")
+        ;
 
     for my $opt (sort keys %opts) {
 
@@ -607,7 +614,7 @@ Test::Moose::More - More tools for testing Moose packages
 
 =head1 VERSION
 
-This document describes version 0.036 of Test::Moose::More - released January 20, 2016 as part of Test-Moose-More.
+This document describes version 0.039 of Test::Moose::More - released October 10, 2016 as part of Test-Moose-More.
 
 =head1 SYNOPSIS
 
@@ -1064,7 +1071,7 @@ L<Test::Moose>
 =head1 BUGS
 
 Please report any bugs or feature requests on the bugtracker website
-https://github.com/RsrchBoy/Test-Moose-More/issues
+L<https://github.com/RsrchBoy/Test-Moose-More/issues>
 
 When submitting a bug or request, please include a test-file or a
 patch to an existing test-file that illustrates the bug or desired
